//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// デバッガのメモリアクセスヘルパー(?)
//

#pragma once

#include "saddr.h"

class DebuggerMD;

enum class MemoryMode : bool
{
	Logical  = false,
	Physical = true,
};

enum class MMULookupMode : bool
{
	False = false,
	True  = true,
	// MemoryMode::Physical ならこの項は意味を持たないので、
	// True でも False でもないと言いたい。
	None  = false,
};

class DebuggerMemoryStream
{
 public:
	// コンストラクタ
	DebuggerMemoryStream(const DebuggerMD *md_);
	// コンストラクタ
	DebuggerMemoryStream(const DebuggerMD *md_, const saddr_t& laddr_,
		MemoryMode access_mode, MMULookupMode lookup_mode);
	// コンストラクタ (lookup 省略)
	DebuggerMemoryStream(const DebuggerMD *md_, const saddr_t& laddr_,
		MemoryMode access_mode)
		: DebuggerMemoryStream(md_, laddr_, access_mode, MMULookupMode::None)
	{
	}
	// コンストラクタ (mode, lookup 省略)
	DebuggerMemoryStream(const DebuggerMD *md_, const saddr_t& laddr_)
		: DebuggerMemoryStream(md_, laddr_, MemoryMode::Logical)
	{
	}

	void InitAddr(const saddr_t& laddr_,
		MemoryMode access_mode, MMULookupMode lookup_mode);

	// 現在のアドレスを表示用に整形して str に代入する。
	// アドレス変換でバスエラーが起きたら false を返す。
	bool FormatAddr(std::string& str);

	// アドレスをリセットする。
	void ResetAddr(uint32 newaddr);

	// アドレスを n 進める(戻す)。
	void Offset(int n);

	// 以下 Peek*()/Fetch*() はいずれも読み込んだデータを返す。
	// (uint64)-1 ならアクセスの結果バスエラー。
	// (uint64)-2 ならアドレス変換でバスエラー。

	// laddr から単位命令語を読み込むが、アドレスは進めない。
	uint64 PeekInst();

	// laddr から単位命令語を読み込んで、アドレスを進める。
	uint64 FetchInst();

	// laddr から1バイト読み込んで、アドレスを進める。
	uint64 Read8();

 public:
	saddr_t laddr {};		// (論理)アドレス
	uint32 paddr {};		// 物理アドレス

 private:
	const DebuggerMD *md {};
	bool lookup {};			// MMU テーブルサーチを行うなら true

	bool translate {};		// アドレス変換が必要なら true
	bool pageover {};		// 256バイト境界をまたいだら true
};
