//
// nono
// Copyright (C) 2022 nono project
// Licensed under nono-license.txt
//

//
// メモリダンプ (モニタ)
//

#pragma once

#include "monitor.h"
#include "debugger_memory.h"

// メモリダンプをする人。基本クラスは CLI (デバッガ) 用。
class Memdump : public Object
{
	using inherited = Object;

 public:
	// フォーマット
	enum Format {
		Byte,
		Word,
		Long,

		M68KMMU_SHORT,
		M68KMMU_LONG,

		M88KMMU,
	};

 public:
	Memdump();
	Memdump(DebuggerMD *md_);
	~Memdump() override;

	void InitMD(DebuggerMD *md_);

	uint32 GetAddr() const	{ return saddr.GetAddr(); }

	// saddr を代入
	void SetAddr(const saddr_t& saddr_);
	// 個別に設定
	void SetAddr(uint32 laddr_);
	void SetSuper(bool super_);
	void SetData(bool data_);
	// モードを設定
	void SetAccessMode(MemoryMode access_mode_);
	void SetLookupMode(MMULookupMode lookup_mode_);

	// アドレスを行単位で加算(減算)
	void OffsetLine(int offset);

	// 表示形式
	Format GetFormat() const { return fmt; }
	void SetFormat(Format fmt_);

	// 1行の表示バイト数を返す
	int GetStride() const { return stride; }

	// メモリの内容を(越権的に)読み出す。ここでは長さは bytes で指定。
	uint64 Peek(uint32 paddr_, int bytes) const;
	// メモリの内容を(越権的に)書き換える。ここでは長さは bytes で指定。
	uint64 Poke(uint32 paddr_, uint32 data, int bytes);
	// このアドレスが Poke 可能かどうかを返す。
	bool CanPoke(uint32 paddr_) const;

	// コンソールに出力。
	void Print(FILE *cons, int row);

 protected:
	std::vector<int> Read(DebuggerMemoryStream&, int bytes);
	std::string DumpChar(const std::vector<int>&);
	std::string DumpHex(const std::vector<int>&);
	std::string Dump68030PageShort(const std::vector<int>&);
	std::string Dump88200Page(const std::vector<int>&);

	DebuggerMD *md {};

	// アドレス等
	saddr_t saddr {};
	MemoryMode access_mode {};
	MMULookupMode lookup_mode {};

	// 表示形式
	Format fmt {};

	// 表示の1行に必要なバイト数
	int stride {};

	static const char * const dtname[4];
};

// メモリダンプをする人。モニタ用。
class MemdumpMonitor : public Memdump
{
	using inherited = Memdump;
 public:
	MemdumpMonitor(int idx_);
	~MemdumpMonitor() override;

 private:
	DECLARE_MONITOR_CALLBACK(MonitorUpdate);

	void MonitorUpdate68030PageShort(TextScreen&, int, const std::vector<int>&);
	void MonitorUpdate88200Page(TextScreen&, int, const std::vector<int>&);

	Monitor monitor { this };
};
