;
; nono
; Copyright (C) 2020 nono project
;
; Redistribution and use in source and binary forms, with or without
; modification, are permitted provided that the following conditions
; are met:
; 1. Redistributions of source code must retain the above copyright
;    notice, this list of conditions and the following disclaimer.
; 2. Redistributions in binary form must reproduce the above copyright
;    notice, this list of conditions and the following disclaimer in the
;    documentation and/or other materials provided with the distribution.
;
; THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
; IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
; OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
; IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
; INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
; BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
; AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
; OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
; OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
; SUCH DAMAGE.

; How to compile:
;  > has fmovem_n.has
;  > hlk fmovem_n.o si_util.o

	.xref	hexstr_long
	.include doscall.mac
	.list
	.cpu	68030

PRINT	.macro	msg
	pea.l	msg(pc)
	DOS	_PRINT
	addq.l	#4,sp
	.endm

SET_FLINE_TRAP	.macro
	move.l	(a5),a4
	lea.l	fline_handler(pc),a1
	move.l	a1,(a5)
	.endm

RESTORE_FLINE_TRAP	.macro
	move.l	a4,(a5)
	.endm

	.text
	.even
<?php
	// レジスタ割り当て
	// d0-d1	work
	// d6	ダイナミックモードでのレジスタ
	// d7	1=検査対象命令でFライン例外がおきた
	//      その後は ok を表示するかどうかのカウント
	// a0-a2	work
	// a3	検査対象命令の次の位置 (Fライン例外から戻るのに使用)
	// a4	元のFライン例外ハンドラ置き場
	// a5	Fライン例外ベクタの位置 $002c
?>
<?php
	// "dam+-i" の各文字は実行できるところ。
	// "." は例外がおきるところ。
	//
	// EA が -(An) なら FP-to-EA (fromFP)の方向のみ可能。
	// EA が (An)+ なら EA-to-FP (toFP)  の方向のみ可能。
	$table = array(
		// mode                           dam+-i
		0/* to FP,   pred, static  */ => "..m+..", // fmovem EA,fp1-fp2
		1/* to FP,   pred, dynamic */ => "..m+..", // fmovem EA,d6
		2/* to FP,   ctrl, static  */ => "..m+..", // fmovem EA,fp1-fp2
		3/* to FP,   ctrl, dynamic */ => "..m+..", // fmovem EA,d6
		4/* from FP, pred, static  */ => "..m.-.", // fmovem fp1-fp2,EA
		5/* from FP, pred, dynamic */ => "..m.-.", // fmovem d6,EA
		6/* from FP, ctrl, static  */ => "..m.-.", // fmovem fp1-fp2,EA
		7/* from FP, ctrl, dynamic */ => "..m.-.", // fmovem d6,EA
	);
	foreach ($table as $mode => $eastr) {
		$ealist = preg_split("//", $eastr, -1, PREG_SPLIT_NO_EMPTY);
		for ($id = 0; $id < 6; $id++) {
			$ea = $ealist[$id];
			if ($ea == ".") {
				// 許されない組み合わせ
				subr_fail($mode, $id);
			} else {
				if ($mode < 4) {
					subr_to($mode, $id);
				} else {
					subr_from($mode, $id);
				}
			}
		}
	}
?>
<?php
// 1行を出力する。引数は printf のように fmt... で指定可能。
// 改行はこちらで付加する。
function out()
{
	$args = func_get_args();
	$fmt = array_shift($args);
	print vsprintf($fmt, $args);
	print "\n";
}

// mode 番号(下2bit)からラベル文字列に。
function mode2label($mode)
{
	$rv .= (($mode & 2) == 0) ? "pred" : "ctrl";
	$rv .= (($mode & 1) == 0) ? "_sta" : "_dyn";
	return $rv;
}

// $mode 番号からターゲットレジスタに。
// Ctrl/PostInc と PreDec とで転送する順番が違う。
// static と dynamic はなんとなく別のマスクを用意しておく。
function mode2fplist($mode)
{
	$fplist = array(
		0/*pred, static  */ => array(6, 5),
		1/*pred, dynamic */ => array(5, 4),
		2/*ctrl, static  */ => array(1, 2),
		3/*ctrl, dynamic */ => array(2, 3),
	);
	return $fplist[$mode & 3];
}

// EA 番号から命令ワードに。n はすべて0
function eanum2word($id)
{
	switch ($id) {
	 case 0:	// D0
		return 0 << 3;
	 case 1:	// A0
		return 1 << 3;
	 case 2:	// (A0)
		return 2 << 3;
	 case 3:	// (A0)+
		return 3 << 3;
	 case 4:	// -(A0)
		return 4 << 3;
	 case 5:	// #imm
		return 0x3c;
	}
}

// EA 番号をラベル文字列に。
function eanum2label($id)
{
	$idstr = array(
		0 => "dn",
		1 => "an",
		2 => "anin",
		3 => "anpi",
		4 => "anpd",
		5 => "imm",
	);
	return $idstr[$id];
}

// EA 番号をニーモニックに。
function eanum2mnem($id, $n = 0)
{
	switch ($id) {
	 case 0:	return "d{$n}";
	 case 1:	return "a{$n}";
	 case 2:	return "(a{$n})";
	 case 3:	return "(a{$n})+";
	 case 4:	return "-(a{$n})";
	}
}

function addfunc($name)
{
	global $funclist;

	$funclist[] = $name;
	out("test_{$name}:");
	out("	PRINT	msg_{$name}");
}

// 結果表示のサブルーチンコールを出力
function out_result($msg, $exp, $act)
{
	out("	pea.l	{$msg}(pc)");
	out("	move.l	{$exp},-(sp)");
	out("	move.l	{$act},-(sp)");
	out("	bsr	result");
	out("	lea.l	12(sp),sp");
}

// 許されない組み合わせのテスト。
// Fライン例外が起きることをチェックする。
function subr_fail($mode, $id)
{
	$modestr = mode2label($mode);
	$ealabel = eanum2label($id);

	$funcname = ($mode & 4) == 0
		? "fmovem_{$ealabel}_to_fpn_{$modestr}"
		: "fmovem_fpn_to_{$ealabel}_{$modestr}";
	addfunc($funcname);

	$w1 = 0xf200 | eanum2word($id);
	$w2 = 0xc060 | ($mode << 11);

	out("	SET_FLINE_TRAP");
	out("	moveq.l	#0,d7");
	out("	lea.l	@f,a3");
	// アセンブルできない可能性があるので命令はバイナリを出力。
	out("	.dc.w	$%04x, $%04x", $w1, $w2);
	out("@@:");
	out("	RESTORE_FLINE_TRAP");

	// 例外が起きるはずなので D7 が非ゼロなら成功(次へ)
	out("	tst.l	d7");
	out("	bne	@f");
	out("	PRINT	msg_fail_notrap");
	out("	rts");
	out("@@:");
	out("	PRINT	msg_oktrap");
	out("	rts");
	out("");
}

// 実行できる組み合わせなら動作確認。EA to FPn
function subr_to($mode, $id)
{
	$modestr = mode2label($mode);
	$ealabel = eanum2label($id);

	$funcname = "fmovem_{$ealabel}_to_fpn_{$modestr}";
	addfunc($funcname);

	$w1 = 0xf200 | eanum2word($id);
	$w2 = 0xc060 | ($mode << 11);
	$predec = ($mode & 2) == 0;
	$dynamic = $mode & 1;

	// FPn を初期化
	out("	lea.l	initdata(pc),a0");
	for ($i = 0; $i < 8; $i++) {
		out("	fmove.x (a0)+,fp{$i}");
	}

	// 読み込み元メモリを用意。EA to FPn 側に -(An) は存在しない。
	out("	lea.l	srcdata(pc),a0");
	if ($dynamic) {
		// ダイナミックモードでの転送マスクもあえて違うものにしておく
		out("	moveq.l	#$30,d6");
	}

	// 検査
	out("	SET_FLINE_TRAP");
	out("	moveq.l	#0,d7");
	out("	lea.l	@f,a3");
	// アセンブルできない可能性があるので命令はバイナリを出力。
	out("	.dc.w	$%04x, $%04x", $w1, $w2);
	out("@@:");
	out("	RESTORE_FLINE_TRAP");

	// 例外は起きないはずなので D7 がゼロなら成功(次へ)
	out("	tst.l	d7");
	out("	beq	@f");
	out("	PRINT	msg_fail_trap");
	out("	rts");
	out("@@:");

	// An を調べる。An はソース
	out("	lea.l	srcdata+%d(pc),a1", ($id == 3)/*(An)+*/ ? 24 : 0);
	out("	cmp.l	a1,a0");
	out("	beq	@f");
	out("	pea.l	msg_fail_addr(pc)");
	out("	move.l	a1,-(sp)");
	out("	move.l	a0,-(sp)");
	out("	bsr	result");
	out("	lea.l	12(sp),sp");
	out("@@:");

	// FPn を取り出してきて比較。
	// f1 なら srcdata[0] と、f2 なら srcdata[1] と。
	// それ以外は initdata[n] と比較する。
	list ($f1, $f2) = mode2fplist($mode);

	out("	lea.l	workarea(pc),a2");
	for ($i = 0; $i < 8; $i++) {
		out("	; compare FP${i}");
		out("	pea.l	msg_fail_fp{$i}(pc)");
		// 比較先は都度違う
		if ($i == $f1) {
			out("	pea.l	srcdata+0(pc)");
		} else if ($i == $f2) {
			out("	pea.l	srcdata+12(pc)");
		} else {
			out("	pea.l	(initdata+%d*12)(pc)", $i);
		}
		out("	fmove.x	fp{$i},(a2)");
		out("	pea.l	(a2)");
		out("	bsr	check_ext");
		out("	lea.l	12(sp),sp");
	}
	out("	tst.l	d7");
	out("	bne	@f");
	out("	PRINT	msg_ok");
	out("@@:");
	out("	rts");
	out("");
}

// 実行できる組み合わせなら動作確認。FPn to EA
function subr_from($mode, $id)
{
	$modestr = mode2label($mode);
	$ealabel = eanum2label($id);

	$funcname = "fmovem_fpn_to_{$ealabel}_{$modestr}";
	addfunc($funcname);

	$w1 = 0xf200 | eanum2word($id);
	$w2 = 0xc060 | ($mode << 11);
	$predec = ($mode & 2) == 0;
	$dynamic = $mode & 1;

	// FPn を初期化
	out("	lea.l	initdata(pc),a0");
	for ($i = 0; $i < 8; $i++) {
		out("	fmove.x	(a0)+,fp{$i}");
	}
	list ($f1, $f2) = mode2fplist($mode);

	// 書き込み先バッファも初期化
	out("	moveq.l	#0,d0");
	out("	moveq.l	#0,d1");
	out("	moveq.l	#0,d7");
	out("	lea.l	workarea(pc),a0");
	out("	movem.l	d0-d1/d7,(a0)");
	out("	movem.l	d0-d1/d7,12(a0)");
	switch ($id) {
	 case 2:	// (An)
		break;
	 case 4:	// -(An)
		out("	lea.l	workarea+24(pc),a0");
		break;
	}
	if ($dynamic) {
		// ダイナミックモードでの転送マスクもあえて違うものにしておく
		out("	moveq.l	#$30,d6");
	}

	// 検査
	out("	SET_FLINE_TRAP");
	out("	lea.l	@f,a3");
	// アセンブルできない可能性があるので命令はバイナリを出力。
	out("	.dc.w	$%04x, $%04x", $w1, $w2);
	out("@@:");
	out("	RESTORE_FLINE_TRAP");

	// 例外は起きないはずなので D7 がゼロなら成功(次へ)
	out("	tst.l	d7");
	out("	beq	@f");
	out("	PRINT	msg_fail_trap");
	out("	rts");
	out("@@:");

	// An を調べる。An は dest。FPn to EA 側に (An)+ は存在しない
	out("	lea.l	workarea(pc),a1");
	out("	cmp.l	a1,a0");
	out("	beq	@f");
	out("	pea.l	msg_fail_addr(pc)");
	out("	move.l	a1,-(sp)");
	out("	move.l	a0,-(sp)");
	out("	bsr	result");
	out("	lea.l	12(sp),sp");
	out("@@:");

	// workarea に書き込まれているのを srcdata と比較
	// workarea の若いアドレスから順に比較するが、
	// FMOVEM fpnlist,-(An) の時 (predec の時ではなく id==4 の時) は
	// 転送順にメモリを下っていくので、比較も下りながら行わないといけない。
	$work = ($id == 4) ? 12 : 0;
	out("	; compare first one");
	out("	pea.l	msg_fail_1st(pc)");
	out("	pea.l	(initdata+%d*12)(pc)", $f1);
	out("	pea.l	%d(a1)", $work);
	out("	bsr	check_ext");
	out("	lea.l	12(sp),sp");
	if ($id == 4)
		$work -= 12;
	else
		$work += 12;
	out("	; compare second one");
	out("	pea.l	msg_fail_2nd(pc)");
	out("	pea.l	(initdata+%d*12)(pc)", $f2);
	out("	pea.l	%d(a1)", $work);
	out("	bsr	check_ext");
	out("	lea.l	12(sp),sp");
	out("");
	out("	tst.l	d7");
	out("	bne	@f");
	out("	PRINT	msg_ok");
	out("@@:");
	out("	rts");
	out("");
}
?>
<?php
// 空リストは正しく0個転送という扱いになる。
// たぶん dynamic でも同様だろうけどもうテストしない。
addfunc("fmovem_nofpn"); ?>
	lea.l	initdata(pc),a0
<?php
	for ($i = 0; $i < 8; $i++) {
		out("	fmove.x (a0)+,fp{$i}");
	}
?>
	SET_FLINE_TRAP
	moveq.l	#0,d7
	lea.l	workarea(pc),a0
	move.l	d7,(a0)
	move.l	d7,4(a0)
	move.l	d7,8(a0)
	lea.l	@f,a3
	.dc.w	$f210, $f000	; fmovem nofpn,(a0)
@@:
	RESTORE_FLINE_TRAP
	tst.l	d7
	beq	@f
	PRINT	msg_fail_trap
	rts
@@:
	pea.l	msg_fail_1st(pc)
	pea.l	zerodata(pc)
	pea.l	(a0)
	bsr	check_ext
	lea.l	12(sp),sp

	tst.l	d7
	bne	@f
	PRINT	msg_ok
@@:
	rts

<?php
// dynamic のレジスタ指定 0rrr_0000 の %0 のところに何か書かれてても無視。
addfunc("fmovem_illegal_dyn"); ?>
	lea.l	initdata(pc),a0
<?php
	for ($i = 0; $i < 8; $i++) {
		out("	fmove.x (a0)+,fp{$i}");
	}
?>
	SET_FLINE_TRAP
	lea.l	srcdata(pc),a0
	moveq.l	#$80,d6
	moveq.l	#0,d7
	lea.l	@f,a3
	.dc.w	$f210, $d86f	; fmovem (a0),d6
@@:
	RESTORE_FLINE_TRAP

	tst.l	d7
	beq	@f
	PRINT	msg_fail_trap
	rts
@@:
	lea.l	workarea(pc),a0
	fmovem.x fp0,(a0)
	pea.l	msg_fail_fp0(pc)
	pea.l	srcdata(pc)
	pea.l	(a0)
	bsr	check_ext
	lea.l	12(sp),sp

	tst.l	d7
	bne	@f
	PRINT	msg_ok
@@:
	rts

<?php
// lea.l $0000002c.l,a5 だと has.x に絶対アドレッシングだと
// 怒られてしまうため仕方なく
?>
start:
	clr.l	-(sp)
	DOS	_SUPER
	movem.l	d6-d7/a2-a5,-(sp)
	moveq.l	#$2c,d0		; alternate of "lea.l $2c,a5"
	move.l	d0,a5		;
<?php
	foreach ($funclist as $name) {
		out("	bsr	test_{$name}");
	}
?>
	movem.l	(sp)+,d6-d7/a2-a5
	DOS	_EXIT

; 結果表示
; スタックには、メッセージ、期待値、実際の値の順で積む。
result:
	link	a6,#0
	movem.l	d0/a0,-(sp)

	move.l	8+8(a6),-(sp)
	DOS	_PRINT
	addq.l	#4,sp

	move.l	8+4(a6),d0
	lea.l	buf(pc),a0
	bsr	hexstr_long
	PRINT	buf
	PRINT	msg_but

	move.l	8+0(a6),d0
	lea.l	buf(pc),a0
	bsr	hexstr_long
	PRINT	buf
	PRINT	msg_crlf

	addq.l	#1,d7	; errcnt
	movem.l	(sp)+,d0/a0
	unlk	a6
	rts

; 拡張精度の比較と結果表示。
; 一致したら何もせず帰る。一致しなければエラーメッセージを表示。
; スタックには、(エラー時の)メッセージ、期待値、実際の値の順で積む
check_ext:
	link	a6,#0
	movem.l	d0/a0-a2,-(sp)

	move.l	8+4(a6),a1
	move.l	8+0(a6),a2
	cmpm.l	(a1)+,(a2)+
	bne	@f
	cmpm.l	(a1)+,(a2)+
	bne	@f
	cmpm.l	(a1)+,(a2)+
	bne	@f
	; equal
	bra	check_ext_done

@@:	; not equal
	move.l	8+8(a6),-(sp)
	DOS	_PRINT
	addq.l	#4,sp

	lea.l	buf(pc),a0
	move.l	8+4(a6),a1
	move.l	(a1)+,d0
	bsr	hexstr_long
	move.b	#'_',(a0)+
	move.l	(a1)+,d0
	bsr	hexstr_long
	move.b	#'_',(a0)+
	move.l	(a1),d0
	bsr	hexstr_long
	PRINT	buf
	PRINT	msg_but

	lea.l	buf(pc),a0
	move.l	8+0(a6),a2
	move.l	(a2)+,d0
	bsr	hexstr_long
	move.b	#'_',(a0)+
	move.l	(a2)+,d0
	bsr	hexstr_long
	move.b	#'_',(a0)+
	move.l	(a2),d0
	bsr	hexstr_long
	PRINT	buf
	PRINT	msg_crlf

	addq.l	#1,d7	; errcnt
check_ext_done:
	movem.l	(sp)+,d0/a0-a2
	unlk	a6
	rts

; Fライン例外ハンドラ
; a3 に戻りアドレスをセットしてあること
; d7 を 1 にして帰る
fline_handler:
	moveq.l	#1,d7
	move.l	a3,2(sp)
	rte

; FPn を埋める初期値
initdata:
	.dc.l	$0, $0, $8
	.dc.l	$0, $0, $1
	.dc.l	$0, $0, $2
	.dc.l	$0, $0, $3
	.dc.l	$0, $0, $4
	.dc.l	$0, $0, $5
	.dc.l	$0, $0, $6
	.dc.l	$0, $0, $7

; ターゲットの FPn に転送するデータ
srcdata:
	.dc.l	$3fff0000, $88112233, $44556677
	.dc.l	$dfff0000, $8899aabb, $ccddeeff

zerodata:
	.dc.l	$0, $0, $0

msg_oktrap:	.dc.b	$9,"ok(trap)",$d,$a,0, 0,0
msg_fail_notrap:	.dc.b	$9,"FAIL: trap expected but not occured",$d,$a,0
msg_fail_trap:	.dc.b	$9,"FAIL: unexpected trap occured",$d,$a,0

msg_ok:		.dc.b	$9,"ok",$d,$a,0
msg_fail_fp0:	.dc.b	$9,"FAIL: fp0 expects ",0
msg_fail_fp1:	.dc.b	$9,"FAIL: fp1 expects ",0
msg_fail_fp2:	.dc.b	$9,"FAIL: fp2 expects ",0
msg_fail_fp3:	.dc.b	$9,"FAIL: fp3 expects ",0
msg_fail_fp4:	.dc.b	$9,"FAIL: fp4 expects ",0
msg_fail_fp5:	.dc.b	$9,"FAIL: fp5 expects ",0
msg_fail_fp6:	.dc.b	$9,"FAIL: fp6 expects ",0
msg_fail_fp7:	.dc.b	$9,"FAIL: fp7 expects ",0
msg_fail_1st:	.dc.b	$9,"FAIL: 1st expects ",0
msg_fail_2nd:	.dc.b	$9,"FAIL: 2nd expects ",0
msg_fail_addr:	.dc.b	$9,"FAIL: an expects ",0
msg_but:	.dc.b	" but ",0
msg_crlf:	.dc.b	$d,$a,0

<?php
	foreach ($funclist as $name) {
		out("msg_{$name}:	.dc.b	\"{$name}\",0");
	}
?>

	.data
	.even
workarea:	.ds.b	24
buf:		.dc.b	64

	.end	start
