//
// nono
// Copyright (C) 2021 nono project
// Licensed under nono-license.txt
//

//
// シグナルスレッド
//

#pragma once

#include "thread.h"
#include <atomic>

class SignalThread : public ThreadDevice
{
	using inherited = ThreadDevice;
	friend void signal_handler(int);

 public:
	SignalThread();
	~SignalThread() override;

	bool StartThread() override;
	void ThreadRun() override;
	void Terminate() override;

	// HostCOM スレッドへのパイプを追加する (HostCOM スレッドから呼ばれる)
	void AddPipe(int fd);
	// HostCOM スレッドへのパイプを削除する
	void DeletePipe();

 private:
	// シグナルを再配送する
	void WriteSignal(int fd, int signo);

	// HostCOM スレッドとのパイプを接続/切断する
	void ChangeCOM(int reqfd);

	// このスレッドにシグナルを投げる
	void Kill(int signo);

	// シグナルをブロック/アンブロックする
	void Mask(int how, int signo);

	// ブロックするシグナル集合
	sigset_t waitset {};

	// HostCOM へのパイプ
	int wpipe {};

	// コマンド(フラグ)
	// 終了フラグは立てるだけ、読むだけなので atomic じゃなくてもいいはず。
	bool request_terminate {};

	// HostCOM からの接続、切断要求。
	// 負数なら要求なし、
	// INT_MAX なら切断を要求中、
	// それ以外ならこのディスクリプタの接続を要求中。
	std::atomic<int> request_wpipe {};
};

extern SignalThread *gSignalThread;
