//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// 各種ビット操作
//

#pragma once

#include "header.h"

extern const uint8 bitrev_table[256];

// 8ビットを左右反転する (定数用)
static constexpr uint8 _rev8(uint8 x)
{
	x = ((x & 0x0f) << 4) | ((x >> 4) & 0x0f);
	x = ((x & 0x33) << 2) | ((x >> 2) & 0x33);
	x = ((x & 0x55) << 1) | ((x >> 1) & 0x55);
	return x;
}

// 8ビットを左右反転する (実行時用)
static inline uint8
bitrev(uint8 x)
{
	return bitrev_table[x];
}

// 右ローテート
static inline uint32
ROR32(uint32 a, int n)
{
	// TODO: 各種コンパイラごとにビルトインがあったりなかったりするらしい
	n &= 31;
	if (n == 0) {
		return a;
	} else {
		return (a >> n) | (a << (32 - n));
	}
}

// mask で指定したいずれかのビットが 0 -> 1 に変化したら true
static constexpr bool
bit_rising(uint before, uint after, uint mask)
{
	// before が 0 で mask が 1 のビットだけを立てる
	// それが 1 で after が 1 なら立ち上がり
	return (~before & mask) & after;
}

// mask で指定したいずれかのビットが 1 -> 0 に変化したら true
static constexpr bool
bit_falling(uint before, uint after, uint mask)
{
	// before が 1 で mask が 1 のビットだけを立てる
	// それが 1 で after が 0 なら立ち下がり
	return (before & mask) & ~after;
}
