//
// nono
// Copyright (C) 2021 nono project
// Licensed under nono-license.txt
//

//
// 色構造体
//

#pragma once

#include "header.h"

//
// 自前のカラー構造体
//
struct Color
{
	uint8 r {};
	uint8 g {};
	uint8 b {};

	Color() {}

	Color(uint8 r_, uint8 g_, uint8 b_) {
		r = r_;
		g = g_;
		b = b_;
	}
};

//
// 自前のカラー<double>構造体
//
struct ColorD
{
	double r {};
	double g {};
	double b {};

	ColorD() {}

	ColorD(double r_, double g_, double b_) {
		r = r_;
		g = g_;
		b = b_;
	}
};

//
// カラー構造体 (xBGR 形式)
//
union ColorXBGR
{
	uint32 xbgr {};
	struct {
#if BYTE_ORDER == LITTLE_ENDIAN
		uint8 r;
		uint8 g;
		uint8 b;
		uint8 x;
#else
		uint8 x;
		uint8 b;
		uint8 g;
		uint8 r;
#endif
	};

	ColorXBGR() { }
	ColorXBGR(int r_, int g_, int b_) {
		r = r_;
		g = g_;
		b = b_;
		x = 0;
	}

	// Color に変換
	Color ToColor() const { return Color(r, g, b); }
};
