//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// メモリストリーム
//

#pragma once

#include "header.h"

// ホストのメモリ領域に対するストリームクラスみたいなもの。
// データの受け渡しはホストエンディアンで、
// アクセスはビッグエンディアンになる。
class MemoryStreamBE
{
 public:
	// コンストラクタ
	MemoryStreamBE() {
	}
	MemoryStreamBE(void *start_) {
		Init(start_);
	}
	// デストラクタ不要

	// 開始ポインタを設定する
	// 現在のポインタ位置は開始ポインタ位置になる
	void Init(void *start_) {
		start = (uint8 *)start_;
		ptr = start;
	}

	// 開始ポインタを取得する
	void *GetStartPtr() const
	{
		return (void *)start;
	}

	// バイトオフセットを設定する
	void SetOffset(uint32 ofs)
	{
		ptr = start + ofs;
	}

	// 現在のバイトオフセットを取得する
	uint32 GetOffset() const
	{
		return (uint32)(ptr - start);
	}

	// 1バイト読み込んでポインタを進める
	uint32 Read8() {
		uint32 data = *ptr++;
		return data;
	}
	// BE で2バイト読み込んでポインタを進める
	uint32 Read16() {
		uint32 data;
		data  = Read8() << 8;
		data |= Read8();
		return data;
	}
	// BE で4バイト読み込んでポインタを進める
	uint32 Read32() {
		uint32 data;
		data  = Read16() << 16;
		data |= Read16();
		return data;
	}

	// 1バイト書き込んでポインタを進める
	void Write8(uint32 data) {
		*ptr++ = data;
	}
	// BE で2バイト書き込んでポインタを進める
	void Write16(uint32 data) {
		Write8(data >> 8);
		Write8(data & 0xff);
	}
	// BE で4バイト書き込んでポインタを進める
	void Write32(uint32 data) {
		Write16(data >> 16);
		Write16(data & 0xffff);
	}

 private:
	uint8 *ptr {};
	uint8 *start {};
};
