//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// 基本オブジェクト
//

// オブジェクトはログ出力機能 (とそのための自身のオブジェクト名) を持つ。

#include "object.h"
#include "logger.h"
#include "mainapp.h"

// コンストラクタ
Object::Object(const std::string& objname_)
{
	// オブジェクト名(ログ表示名)を設定。
	objname = objname_;

	// コンストラクタで指定した場合はエイリアスにも登録しておく。
	AddAlias(objname_);

	// 自分自身をリストに繋ぐ
	gMainApp.AddObject(this);
}

// デストラクタ
Object::~Object()
{
	// 自分自身をリストから削除する
	gMainApp.DeleteObject(this);
}

// ログレベルを設定する。
void
Object::SetLogLevel(int loglevel_)
{
	loglevel = loglevel_;
}

// メッセージ表示
// 常に表示 (あらかじめ loglevel の評価をした後で呼び出す用)
void
Object::putmsgn(const char *fmt, ...) const
{
	char buf[1024];
	va_list ap;
	int len;

	// 名前だけ生表示されても分かりづらいので括弧でもつけておく
	len = snprintf(buf, sizeof(buf), "(%s) ", GetName().c_str());

	va_start(ap, fmt);
	vsnprintf(buf + len, sizeof(buf) - len, fmt, ap);
	va_end(ap);

	WriteLog(buf);
}

// ログ表示 (継承側で用意すること)
void
Object::putlogn(const char *fmt, ...) const
{
	PANIC("Object::putlogn called");
}

// ロガーに出力する共通部分
void
Object::WriteLog(const char *buf) const
{
	logger->Write(buf);
}
