//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#pragma once

#include "bus.h"

// 基本アクセス
extern uint32 m68030_fetch_8(m68kcpu *cpu);
extern uint32 m68030_fetch_16(m68kcpu *cpu);
extern uint32 m68030_fetch_32(m68kcpu *cpu);
extern uint32 m68030_read_8(m68kcpu *cpu, uint32 addr);
extern uint32 m68030_read_16(m68kcpu *cpu, uint32 addr);
extern uint32 m68030_read_32(m68kcpu *cpu, uint32 addr);
extern void m68030_write_8(m68kcpu *cpu, uint32 addr, uint32 data);
extern void m68030_write_16(m68kcpu *cpu, uint32 addr, uint32 data);
extern void m68030_write_32(m68kcpu *cpu, uint32 addr, uint32 data);
// FC 指定
extern uint32 m68030_read_8_fc(m68kcpu *cpu, uint32 addr);
extern uint32 m68030_read_16_fc(m68kcpu *cpu, uint32 addr);
extern uint32 m68030_read_32_fc(m68kcpu *cpu, uint32 addr);
extern void m68030_write_8_fc(m68kcpu *cpu, uint32 addr, uint32 data);
extern void m68030_write_16_fc(m68kcpu *cpu, uint32 addr, uint32 data);
extern void m68030_write_32_fc(m68kcpu *cpu, uint32 addr, uint32 data);

// 基本アクセスルーチンを使った便利サブルーチン
static inline void
m68030_push_16(m68kcpu *cpu, uint32 data)
{
	RegA(7) -= 2;
	m68030_write_16(cpu, RegA(7), data);
}
static inline void
m68030_push_32(m68kcpu *cpu, uint32 data)
{
	RegA(7) -= 4;
	m68030_write_32(cpu, RegA(7), data);
}

static inline uint32
m68030_pop_16(m68kcpu *cpu)
{
	uint32 data = m68030_read_16(cpu, RegA(7));
	RegA(7) += 2;
	return data;
}

static inline uint32
m68030_pop_32(m68kcpu *cpu)
{
	uint32 data = m68030_read_32(cpu, RegA(7));
	RegA(7) += 4;
	return data;
}
