//
// nono
// Copyright (C) 2022 nono project
// Licensed under nono-license.txt
//

//
// ローダ
//

#pragma once

#include "object.h"
#include "iodevstream.h"

class LoadInfo;
class MainRAMDevice;

// RAM から呼ばれてファイルを読み込むだけのヘルパー。
class BootLoader : public Object
{
	using inherited = Object;
 public:
	BootLoader(MainRAMDevice *ram_);
	~BootLoader() override;

	// path で指定されたホストファイルをロードする
	bool LoadFromFile(LoadInfo *);
	// data, size で指定されたバッファをロードする
	bool LoadFromData(LoadInfo *);

 private:
	bool LoadGzFile(LoadInfo *);
	bool Load_aout(LoadInfo *);
	bool Load_elf32(LoadInfo *);
	bool Load_elf32_exec(LoadInfo *);
	bool Load_elf32_rel(LoadInfo *);
	bool Load_elf32_sym(LoadInfo *, std::string&);
	void WriteFromHost(IODeviceStream&, const void *, int);
	void readfile_global(uint32, const void *, int);

	MainRAMDevice *ram {};
};

// 実行ファイルを読み込む際の受け渡しパラメータ
class LoadInfo {
 public:
	LoadInfo();
	LoadInfo(const char *path_);
	LoadInfo(const char *path_, const uint8 *data_, size_t size_);

	// ファイルから読み込む時はファイルパスを指定する。
	// メモリから読み込む時はエラーの時に表示される名前を指定する。
	const char *path {};

	// データ本体とそのサイズ。メモリから読み込む時には指定する。
	// ファイルから読み込む時は内部で使用。
	const uint8 *data {};
	size_t size {};

	// RAM にロードした先頭アドレス。
	uint32 start {};

	// RAM にロードしたファイルのエントリポイント。
	uint32 entry {};

	// RAM にロードしたカーネル本体の後のシンボル開始アドレス。
	uint32 sym {};

	// RAM にロードした最終アドレス(の次のアドレス)。
	uint32 end {};
};
