//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// メインバス
//

#pragma once

#include "device.h"
#include <array>

class MainbusDevice : public IOContainerDevice
{
	using inherited = IOContainerDevice;
 public:
	MainbusDevice();
	virtual ~MainbusDevice() override;

	// こっちは形式的に用意してあるが vm_phys_*() 関数を使用のこと。
	uint64 Read8(uint32 addr) override;
	uint64 Read16(uint32 addr) override;
	uint64 Read32(uint32 addr) override;
	uint64 Write8(uint32 addr, uint32 data) override;
	uint64 Write16(uint32 addr, uint32 data) override;
	uint64 Write32(uint32 addr, uint32 data) override;
	uint64 Peek8(uint32 addr) override;

	// デバイスをセットする
	static void SetDevice(int idx, IODevice *dev);

	// デバイステーブルをコピーする
	static void CopyDevtable(int dst, int src, int len);

	static std::array<IODevice *, 256> devtable;

	static int direct_ram_size;

 protected:
	IODevice *GetDevice(uint32 addr) const override;
};

extern MainbusDevice *gMainbus;

// メインバスアクセス。
// addr はそれぞれアラインされていること。
extern uint64 vm_phys_read_8(uint32 addr);
extern uint64 vm_phys_read_16(uint32 addr);
extern uint64 vm_phys_read_32(uint32 addr);
extern uint64 vm_phys_write_8(uint32 addr, uint32 data);
extern uint64 vm_phys_write_16(uint32 addr, uint32 data);
extern uint64 vm_phys_write_32(uint32 addr, uint32 data);
extern uint64 vm_phys_peek_8(uint32 addr);
extern uint64 vm_phys_peek_32(uint32 addr);
