//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// M88200 CMMU の I/O デバイス部分
//

#pragma once

#include "device.h"
#include "m88200.h"

class CMMUDevice : public IODevice
{
	using inherited = IODevice;

	// レジスタオフセット
	static const uint32 IDR		= 0x000;
	static const uint32 SCR		= 0x004;
	static const uint32 SSR		= 0x008;
	static const uint32 SAR		= 0x00c;
	static const uint32 SCTR	= 0x104;
	static const uint32 PFSR	= 0x108;
	static const uint32 PFAR	= 0x10c;

	static const uint32 SAPR	= 0x200;
	static const uint32 UAPR	= 0x204;

	static const uint32 BWP0	= 0x400;
	static const uint32 BWP7	= 0x41c;
	static const uint32 CDP0	= 0x800;
	static const uint32 CDP3	= 0x80c;
	static const uint32 CTP0	= 0x840;
	static const uint32 CTP3	= 0x84c;

	static const uint32 CSSP	= 0x880;

 public:
	CMMUDevice();
	virtual ~CMMUDevice() override;

	bool Init() override;

	uint64 Read32(uint32 addr) override;
	uint64 Write32(uint32 addr, uint32 data) override;
	uint64 Peek8(uint32 addr) override;

 private:
	// アドレスデコーダ
	static uint32 Decoder(uint32 addr);

	// 指定された ID の CMMU(m88200) を返す。なければ NULL を返す。
	m88200 *GetCMMU(uint id) const {
		if (id >= countof(cmmus) || cmmus[id] == NULL) {
			return NULL;
		}
		return cmmus[id];
	}
	// CMMU へのリンク (最大 4 MPU)
	m88200 *cmmus[8] {};

	// Peek8() の下請け
	uint32 Peek32(uint32 addr) const;
};

extern CMMUDevice *gCMMU;
