//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// Ethernet 基本クラス
//

//          IODevice
//              |
//              v
//        EthernetDevice (イーサネットとしての共通部分)
//              |
//     +--------+-------+
//     v                v
// LanceDevice     RTL8019Device
// (LUNA, AM7990)  (X68000, RTL8019)

#pragma once

#include "device.h"
#include "macaddr.h"

class HostNetDevice;

class EthernetDevice : public IODevice
{
	using inherited = IODevice;
 protected:
	EthernetDevice(const std::string& objname_);
 public:
	virtual ~EthernetDevice() override;

	bool Create() override;

	// ホストデバイスを取得する。Create() 完了後から使用可能。
	// (ステータスパネルが呼び出す)
	HostNetDevice *GetHostNet() const { return hostnet.get(); }

	// 設定から MAC アドレスを取得するヘルパー関数。
	// (MAC アドレスを保持するデバイスが呼ぶ)
	static bool GetConfigMacAddr(macaddr_t *dst, bool accept_rom);

 protected:
	// ホストスレッドからの受信通知
	void HostRxCallback();

	std::unique_ptr<HostNetDevice> hostnet /*{}*/;
};

extern EthernetDevice *gEthernet;
