//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// GVRAM
//

// GVRAM はワード単位でホストバイトオーダ配置なので、
// バイトアクセス時は HB() マクロを使用のこと。

#include "gvram.h"
#include "mpu.h"
#include "videoctlr.h"

// グローバル参照用
GVRAMDevice *gGVRAM;

// コンストラクタ
GVRAMDevice::GVRAMDevice()
	: inherited("GVRAM")
{
	uint devlen = 0x20'0000;	// XXX とりあえず

	mem.reset(new uint8[devlen]);
}

// デストラクタ
GVRAMDevice::~GVRAMDevice()
{
	gGVRAM = NULL;
}

uint64
GVRAMDevice::Read8(uint32 addr)
{
	gMPU->AddCycle(9);	// Inside/Out p.135
	uint32 offset = addr - baseaddr;
	return mem[HB(offset)];
}

uint64
GVRAMDevice::Read16(uint32 addr)
{
	gMPU->AddCycle(9);	// Inside/Out p.135
	uint32 offset = addr - baseaddr;
	return *(uint16 *)&mem[offset];
}

uint64
GVRAMDevice::Write8(uint32 addr, uint32 data)
{
	gMPU->AddCycle(9);	// Inside/Out p.135
	uint32 offset = addr - baseaddr;
	mem[HB(offset)] = data;
	return 0;
}

uint64
GVRAMDevice::Write16(uint32 addr, uint32 data)
{
	gMPU->AddCycle(9);	// Inside/Out p.135
	uint32 offset = addr - baseaddr;
	*(uint16 *)&mem[offset] = data;
	return 0;
}

uint64
GVRAMDevice::Peek8(uint32 addr)
{
	return mem[HB(addr - baseaddr)];
}
