//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// メインメモリ
//

#pragma once

#include "device.h"
#include "bootloader.h"

class MainRAMDevice : public IODevice
{
	using inherited = IODevice;
 public:
	MainRAMDevice();
	virtual ~MainRAMDevice() override;

	bool Init() override;
	void ResetHard(bool poweron) override;

	uint64 Read8(uint32 addr) override;
	uint64 Read16(uint32 addr) override;
	uint64 Read32(uint32 addr) override;
	uint64 Write8(uint32 addr, uint32 data) override;
	uint64 Write16(uint32 addr, uint32 data) override;
	uint64 Write32(uint32 addr, uint32 data) override;
	uint64 Peek8(uint32 addr) override;
	uint64 Poke8(uint32 addr, uint32 data) override;

	// RAM 容量(バイト単位)を取得
	int GetSize() const	{ return ram_size; }
	// RAM 容量(MB単位)を取得
	int GetSizeMB() const { return (uint)ram_size / 1024 / 1024; }

	// アクセスウェイトを設定
	void SetWait(uint32 wait);

	// path で指定されたホストファイルをロードする
	bool LoadFromFile(LoadInfo *);
	// data, size で指定されたバッファをロードする
	bool LoadFromData(LoadInfo *);

	static std::unique_ptr<uint8[]> mainram;

 private:
	// RAM 容量(バイト単位)。ram[] の確保したバイト数
	int ram_size {};

	uint32 read_wait {};
	uint32 write_wait {};
};

extern MainRAMDevice *gMainRAM;
