//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// MPU 共通部
//

#pragma once

#include "device.h"
#include "event.h"
#include "message.h"

class MPUDevice : public Device
{
	using inherited = Device;
 public:
	MPUDevice(const std::string& objname_);
	virtual ~MPUDevice() override;

	bool Init() override;
	void PowerOff() override;

	// 現在実行中の命令の PC を取得
	virtual uint32 GetPPC() const = 0;

	// 現在の VBR を取得
	virtual uint32 GetVBR() const = 0;

	// 現在アクセス中の論理アドレス、物理アドレスを取得。
	// Laddr == Paddr が MMU オフなのか PA=VA で運用中なのかの区別は
	// ここでは付かない。
	virtual uint32 GetLaddr() const = 0;
	virtual uint32 GetPaddr() const = 0;

	// MPU クロック [kHz] を取得する。
	int GetClockSpeed() const { return clock_khz; }

	// MPU クロック [nsec] を取得する。
	uint64 GetClock_nsec() const { return clock_nsec; }

	// アクセスウェイトを加算
	virtual void AddCycle(int32 cycle) = 0;

	// 割り込み発生を MPU に通知
	virtual void Interrupt(int level) = 0;

	// MPU からのアクセスをエミュレートする。
	// アドレスは論理アドレス。
	// 空間は現在のデータ空間。
	// アクセスエラーは (int64)-1 を返す。
	virtual uint64 Read8(uint32 addr) = 0;
	virtual uint64 Read16(uint32 addr) = 0;
	virtual uint64 Read32(uint32 addr) = 0;
	virtual uint64 Write8(uint32 addr, uint32 data) = 0;
	virtual uint64 Write16(uint32 addr, uint32 data) = 0;
	virtual uint64 Write32(uint32 addr, uint32 data) = 0;

 protected:
	void TraceMessage(MessageID, uint32);
	virtual void SetTrace(bool trace_on) = 0;

	int clock_khz {};		// MPU クロック [kHz]
	uint64 clock_nsec {};	// MPU クロック [nsec]

	bool resetting {};		// リセット処理中フラグ

	// 1命令実行イベント
	Event exec_event { this };

	// Round Mode (モニタ表示用)
	static const char * const rmstr[4];
};

extern MPUDevice *gMPU;
