//
// nono
// Copyright (C) 2022 nono project
// Licensed under nono-license.txt
//

//
// NEWS の各種制御ポート(?)
//

#pragma once

#include "device.h"
#include "event.h"
#include <array>

class NewsInterrupt;

class NewsCtlrDevice : public IODevice
{
	using inherited = IODevice;

 public:
	// LED 種別
	static const uint LED_GREEN		= 0x01;
	static const uint LED_ORANGE	= 0x02;

 public:
	NewsCtlrDevice();
	~NewsCtlrDevice() override;

	bool Init() override;
	void ResetHard(bool poweron) override;

	uint64 Read8(uint32 addr) override;
	uint64 Write8(uint32 addr, uint32 data) override;
	uint64 Peek8(uint32 addr) override;

	int GetLED() const { return led; }

 private:
	uint8 GetIDROM(uint32 addr) const;
	uint8 GetDIPSW() const;
	void WriteLED(uint32 data);
	void WriteTimer(uint32 data);
	void TimerCallback(Event&);
	void ChangeInterrupt();

	// 割り込みコントローラ
	NewsInterrupt *interrupt {};

	// 256 x 4bit ROM (ここではデータをバイト単位で保持する)
	std::array<uint8, 128> oidrom {};

	int led {};
	// タイマー
	bool timer_intr {};
	bool timer_enable {};
	// DIP-SW
	std::array<bool, 8> dipsw {};

	Event timer_event { this };
};

extern NewsCtlrDevice *gNewsCtlr;
