//
// nono
// Copyright (C) 2022 nono project
// Licensed under nono-license.txt
//

//
// NEWS のフレームバッファ(予定地)
//

#include "newsfb.h"

// グローバル参照用
NewsfbDevice *gNewsfb;

// コンストラクタ
NewsfbDevice::NewsfbDevice()
	: inherited("Newsfb")
{
	console.Init(80, 32, TextScreen::Console);

	monitor.func = ToMonitorCallback(&NewsfbDevice::MonitorUpdate);
	monitor.SetSize(console.GetCol(), console.GetRow());
	monitor.Regist(ID_MONITOR_ROMCONS);
}

// デストラクタ
NewsfbDevice::~NewsfbDevice()
{
	gNewsfb = NULL;
}

void
NewsfbDevice::ResetHard(bool poweron)
{
	console.Clear();
}

void
NewsfbDevice::Putc(int ch)
{
	if (ch == '\t') {
		// タブはここでスペースに展開
		int x = console.GetX();
		int next = roundup(x, 8);
		for (; x < next; x++) {
			console.Putc(' ');
		}
		return;
	}
	console.Putc(ch);
}

void
NewsfbDevice::MonitorUpdate(Monitor *, TextScreen& screen)
{
	int row = console.GetRow();
	int col = console.GetCol();
	const auto& src = console.GetBuf();

	int s = 0;
	for (int y = 0; y < row; y++) {
		for (int x = 0; x < col; x++) {
			screen.Putc(x, y, src[s++]);
		}
	}
}
