//
// nono
// Copyright (C) 2021 nono project
// Licensed under nono-license.txt
//

//
// ABORT/INTERRUPT スイッチによる NMI 機構デバイス
//

#include "nmi.h"
#include "interrupt.h"
#include "mainapp.h"
#include "scheduler.h"

// グローバル参照用
NMIDevice *gNMI;

// コンストラクタ
NMIDevice::NMIDevice()
	: inherited("NMI")
{
	event.func = ToEventCallback(&NMIDevice::NegateCallback);
	event.time = 100_msec;
	event.Regist("NMI");
}

// デストラクタ
NMIDevice::~NMIDevice()
{
	gNMI = NULL;
}

// 初期化
bool
NMIDevice::Init()
{
	gScheduler->ConnectMessage(MessageID::NMI, this,
		ToMessageCallback(&NMIDevice::AssertCallback));
	return true;
}

// NMI ボタンを押す (GUI から呼ばれる)
void
NMIDevice::PressNMI()
{
	gScheduler->SendMessage(MessageID::NMI);
}

// NMI 信号線をアサートする
void
NMIDevice::AssertNMI()
{
	gInterrupt->AssertINT(this);
}

// NMI 信号線をネゲートする
void
NMIDevice::NegateNMI()
{
	gInterrupt->NegateINT(this);
}

// NMI アサートのメッセージコールバック
void
NMIDevice::AssertCallback(MessageID msgid, uint32 arg)
{
	AssertNMI();

	if (gMainApp.Has(VMCap::LUNA)) {
		// LUNA(-I) は誰がいつどうネゲートしているのか分からないが
		// コードが能動的にネゲートしているようには見えないので、
		// とりあえず適当な時間経過後に自動的にネゲートしておいてみる。
		gScheduler->RestartEvent(event);
	}
}

// NMI ネゲートのイベントコールバック
void
NMIDevice::NegateCallback(Event& ev)
{
	NegateNMI();
}
