//
// nono
// Copyright (C) 2022 nono project
// Licensed under nono-license.txt
//

//
// 電源周り
//

#pragma once

#include "device.h"
#include "event.h"
#include "message.h"

class PowerDevice : public Device
{
	using inherited = Device;
 public:
	PowerDevice();
	~PowerDevice() override;

	// 電源ボタンを押す
	void PowerButton();

	// リセットを指示
	// (どのスレッドから呼んでもよい)
	void MakeResetHard();

	// リスタートを指示
	// (どのスレッドから呼んでもよい)
	void MakeRestart();

	// 電源が実際にオンなら true を返す。
	bool IsPower() const { return ispower; }

	// 電源 LED 点灯なら true を返す。
	bool GetPowerLED() const { return led; }

	// X68030 では電源ボタンが押されていれば true を返す。
	// LUNA では意味を持たないので呼ばないこと。
	bool IsPowerPressed() const { return power_button; }

	void SetAlarmOut(bool alarm_out_);
	void SetSystemPowerOn(bool poweron);
 private:
	void DoPowerOn();
	void DoResetHard();
	void DoPowerOff(MessageID);
	void DoPowerButton();

	// メッセージコールバック
	void ResetCallback(MessageID, uint32);
	void PowerOffCallback(MessageID, uint32);
	void PowerButtonCallback(MessageID, uint32);

	void PowerCallback(Event&);

	void Change();

	// 電源オンなら true
	bool ispower {};

	// 電源 LED
	bool led {};

	// 電源ボタン状態
	bool power_button {};

	// ALARM_OUT 信号状態
	bool alarm_out {};

	// システム内の電源オン信号に相当
	bool system_poweron {};

	// 電源オフイベント
	Event event { this };
};

extern PowerDevice *gPower;
