//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// PROM (LUNA-I, LUNA-88K 共通)
//

#pragma once

#include "rom.h"

class PROMDevice;

class PROM0Device : public IODevice
{
	using inherited = IODevice;
 public:
	PROM0Device();
	virtual ~PROM0Device() override;

	bool Init() override;
	void ResetHard(bool poweron) override;

	uint64 Read8(uint32 addr) override;
	uint64 Read16(uint32 addr) override;
	uint64 Read32(uint32 addr) override;
	uint64 Write8(uint32 addr, uint32 data) override;
	uint64 Write16(uint32 addr, uint32 data) override;
	uint64 Write32(uint32 addr, uint32 data) override;
	uint64 Peek8(uint32 addr) override;

 private:
	// アドレスデコーダ
	static uint32 Decoder(uint32 addr);

	// 書き込みの共通部分
	uint64 Write();

	IODevice *prom {};
};

class PROMDevice : public ROMDevice
{
	using inherited = ROMDevice;
 public:
	PROMDevice();
	virtual ~PROMDevice() override;

	bool Init() override;

	int GetROMVer() const { return romver; }

 private:
	bool InitLuna1();
	bool InitLuna88k();

	// PROM バージョン。
	// 100倍した整数形式で持つ。0 ならバージョン不明。
	int romver {};
};

extern PROM0Device *gPROM0;
extern ROMDevice *gPROM;
