//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// SRAM
//

#pragma once

#include "device.h"
#include "mappedfile.h"

class SRAMDevice : public IODevice
{
	using inherited = IODevice;

	static const uint32 baseaddr = 0xed0000;

	// Pluto-X によるホストファイル起動アドレス
	static const uint32 plutoaddr = 0xeac000;

 public:
	SRAMDevice();
	virtual ~SRAMDevice() override;

	bool Init() override;
	void ResetHard(bool poweron) override;

	uint64 Read8(uint32 addr) override;
	uint64 Read16(uint32 addr) override;
	uint64 Write8(uint32 addr, uint32 data) override;
	uint64 Write16(uint32 addr, uint32 data) override;
	uint64 Peek8(uint32 addr) override;
	uint64 Poke8(uint32 addr, uint32 data) override;

	void WriteEnable(bool value);

	// SRAM の指定の位置の値を副作用なく読み出す
	uint32 Get8(uint32 offset) const;
	uint32 Get16(uint32 offset) const;
	uint32 Get32(uint32 offset) const;

	// SRAM $ed0008.L RAM 容量を返す (ホストファイル起動用)
	uint32 GetRAMSize() const { return Get32(0x0008); }

	// SRAM $ed000c.L ROM 起動アドレスを返す (romemu 用)
	uint32 GetROMAddr() const { return Get32(0x000c); }

 private:
	// RAM 容量欄を設定値に同期させる。
	void SyncRAMSize();

	// アドレスデコーダ
	uint64 Decoder(uint32 addr) const;

	// ファイル
	std::string filename {};
	MappedFile file {};
	uint8 *mem {};

	bool writeable {};
};

extern SRAMDevice *gSRAM;
