//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// サブメモリ
//

// SubRAM はバイト配置とする。

#include "subram.h"

// グローバル参照用
SubRAMDevice *gSubRAM;

// コンストラクタ
SubRAMDevice::SubRAMDevice(uint ramsize_kb)
	: inherited("SubRAM")
{
	ramsize = ramsize_kb * 1024;

	ram.reset(new uint8[ramsize]);
}

// デストラクタ
SubRAMDevice::~SubRAMDevice()
{
	gSubRAM = NULL;
}

// アドレスデコーダ
inline uint64
SubRAMDevice::Decoder(uint32 addr) const
{
	// たぶん 128KB というかサイズ分で折り返しイメージが見えるはず。要確認。
	return addr % ramsize;
}

uint64
SubRAMDevice::Read8(uint32 addr)
{
	uint32 offset = Decoder(addr);
	uint32 data = ram[offset];
	return data;
}

uint64
SubRAMDevice::Read16(uint32 addr)
{
	uint32 offset = Decoder(addr);
	uint32 data;
	data  = (uint32)ram[offset + 0] << 8;
	data |= (uint32)ram[offset + 1];
	return data;
}

uint64
SubRAMDevice::Read32(uint32 addr)
{
	uint32 offset = Decoder(addr);
	uint32 data;
	data  = (uint32)ram[offset + 0] << 24;
	data |= (uint32)ram[offset + 1] << 16;
	data |= (uint32)ram[offset + 2] << 8;
	data |= (uint32)ram[offset + 3];
	return data;
}

uint64
SubRAMDevice::Write8(uint32 addr, uint32 data)
{
	putlog(3, "$%08x.B <- $%02x", addr, data);

	uint32 offset = Decoder(addr);
	ram[offset] = data;
	return 0;
}

uint64
SubRAMDevice::Write16(uint32 addr, uint32 data)
{
	putlog(3, "$%08x.W <- $%04x", addr, data);

	uint32 offset = Decoder(addr);
	ram[offset + 0] = (data >> 8) & 0xff;
	ram[offset + 1] =  data       & 0xff;
	return 0;
}

uint64
SubRAMDevice::Write32(uint32 addr, uint32 data)
{
	putlog(3, "$%08x.L <- $%08x", addr, data);

	uint32 offset = Decoder(addr);
	ram[offset + 0] =  data >> 24;
	ram[offset + 1] = (data >> 16) & 0xff;
	ram[offset + 2] = (data >> 8)  & 0xff;
	ram[offset + 3] =  data        & 0xff;
	return 0;
}

uint64
SubRAMDevice::Peek8(uint32 addr)
{
	uint32 offset = Decoder(addr);
	return ram[offset];
}

uint64
SubRAMDevice::Poke8(uint32 addr, uint32 data)
{
	if ((int32)data >= 0) {
		uint32 offset = Decoder(addr);
		ram[offset] = data;
	}
	return 0;
}
