//
// nono
// Copyright (C) 2021 nono project
// Licensed under nono-license.txt
//

//
// スレッドを持つデバイス
//

#include "thread.h"
#include "mythread.h"

// コンストラクタ
ThreadDevice::ThreadDevice(const std::string& objname_)
	: inherited(objname_)
{
	// スレッド名はデフォルトでこのオブジェクトと同じ名前にしておく。
	// 継承側コンストラクタ等で SetThreadName() で変更してもよい。
	SetThreadName(GetName().c_str());
}

// デストラクタ
ThreadDevice::~ThreadDevice()
{
	// ここで virtual の Terminate() は呼べないので、
	// 継承側のデストラクタで TerminateThread() をそれぞれ呼ぶこと。
}

// スレッド名を設定する
// (ポインタしか管理しないので実体は呼び出し側で責任を持つこと)
void
ThreadDevice::SetThreadName(const char *threadname_)
{
	assert(threadname_);
	threadname = threadname_;
}

// スレッド開始
bool
ThreadDevice::StartThread()
{
	auto func = [this]() {
		PTHREAD_SETNAME(threadname);
		std::lock_guard<std::mutex> lock_sub(this->thread_starter);
		this->ThreadRun();
	};

	// スレッド起動
	std::lock_guard<std::mutex> lock(thread_starter);

	thread.reset(new std::thread(func));
	if ((bool)thread == false) {
		return false;
	}
	return true;
}

// スレッドに終了を要求し、その終了を待つ。
// スレッド外から呼ぶこと。
void
ThreadDevice::TerminateThread()
{
	if ((bool)thread) {
		Terminate();

		if (thread->joinable()) {
			thread->join();
		}

		thread.reset();
	}
}
