//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// TVRAM
//

#pragma once

#include "planevram.h"

class TVRAMDevice : public PlaneVRAMDevice
{
	using inherited = PlaneVRAMDevice;

	static const int baseaddr = 0xe00000;

	// プレーン数は4枚固定
	static const int NPLANE = 4;

	struct tvram {
		bool sa;				// 同時アクセス
		bool ap[NPLANE];		// 同時アクセスプレーン
		bool cp[NPLANE];		// ラスターコピー対象
		bool men;				// アクセスマスクイネーブル
		uint16 mask;			// アクセスマスク
	};

 public:
	TVRAMDevice();
	virtual ~TVRAMDevice() override;

	bool Init() override;
	void ResetHard(bool poweron) override;

	uint64 Read8(uint32 addr) override;
	uint64 Read16(uint32 addr) override;
	uint64 Write8(uint32 addr, uint32 data) override;
	uint64 Write16(uint32 addr, uint32 data) override;
	uint64 Peek8(uint32 addr) override;
	uint64 Poke8(uint32 addr, uint32 data) override;

	void UpdateInfo(TextScreen&, int x, int y) const override;

	void SetAccessPlane(uint16 data);
	void SetAccessMask(uint16 data);
	void SetScrollX(int x);
	void SetScrollY(int y);
	void RasterCopy(int src_ras, int dst_ras);

 private:
	// アドレスデコーダ
	uint64 Decoder(uint32 addr) const;

	inline void SetDirty(uint32 offset);

	// X 方向にはみ出したときに対応する仮想画面の Y 座標を返す。
	int GetWrapY(int y) const override;

	struct tvram tvram {};
};

#define gTVRAM	(dynamic_cast<TVRAMDevice *>(gPlaneVRAM))
