//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// ビデオコントローラ
//

#pragma once

#include "device.h"
#include "color.h"
#include <array>

class VideoCtlrDevice : public IODevice
{
	using inherited = IODevice;

	static const uint32 baseaddr = 0xe82000;

 public:
	VideoCtlrDevice();
	virtual ~VideoCtlrDevice() override;

	void ResetHard(bool poweron) override;

	// 加工済みテキストパレットの先頭アドレスを取得
	// レンダラとかからの参照用
	const std::vector<ColorXBGR>& GetHostPalette() const { return hostcolor; }

	// 生パレットを uint32 形式にしたものを返す (GUI 用)
	const std::vector<uint32> GetIntPalette() const;

	uint64 Read8(uint32 addr) override;
	uint64 Read16(uint32 addr) override;
	uint64 Write8(uint32 addr, uint32 data) override;
	uint64 Write16(uint32 addr, uint32 data) override;
	uint64 Peek8(uint32 addr) override;
	uint64 Poke8(uint32 addr, uint32 data) override;

 private:
	inline uint32 Decoder(uint32 addr) const;
	inline uint Offset2NR(uint32 offset) const;

	uint32 GetNR(uint reg) const;
	void SetNR(uint reg, uint32 data);
	void MakePalette();

	// ワードレジスタ R0, R1, R2 は内部では nr0 .. nr5 というバイトレジスタ
	// として扱う。
	std::array<uint8, 6> nreg {};

	// メモリを確保する時のサイズ計算のため型を uint8 にしているが、
	// 運用はワードでホストバイトオーダー。
	std::array<uint8, 1024> palette {};

	// ホスト形式のテキストパレット (レンダラ用に加工したもの)
	std::vector<ColorXBGR> hostcolor {};
};

extern VideoCtlrDevice *gVideoCtlr;
