//
// nono
// Copyright (C) 2022 nono project
// Licensed under nono-license.txt
//

//
// VM (NEWS)
//

#include "vm_news.h"
#include "bus.h"
#include "busio.h"
#include "config.h"
#include "dipsw.h"
#include "interrupt.h"
#include "mainapp.h"
#include "mainram.h"
#include "mpu680x0.h"
#include "newsctlr.h"
#include "newsfb.h"
#include "newsio.h"
#include "prom.h"
#include "renderer.h"
#include "romemu_news.h"
#include "scheduler.h"

// コンストラクタ
VM_NEWS::VM_NEWS()
{
	// 機種固有パラメータ
	// MPU クロックの初期値は機種ごとに異なる。単位は MHz
	gConfig->SetDefault("mpu-clock", 25);
	// RAM 初期値は?
	gConfig->SetDefault("ram-size", 16);
	// RTC epoch は 1900年。
	gConfig->SetDefault("rtc-epoch-year", 1900);

	NEWDV(MPU, new MPU680x0Device());
	NEWDV(Interrupt, new NewsInterrupt());
	NEWDV(PROM, new NewsROMEmuDevice());
	NEWDV(MainRAM, new MainRAMDevice());
	NEWDV(NewsCtlr, new NewsCtlrDevice());
	NEWDV(NewsIO, new NewsIODevice());

	NEWDV(Renderer, new NewsRenderer());
	NEWDV(Newsfb, new NewsfbDevice());
	NEWDV(Dipsw, new NewsDipswDevice());

	// 全域をバスエラーで埋める。
	// XXX バスエラーが起きるかどうかは知らないが
	for (int i = 0; i < gMainbus->devtable.size() / 4; i++) {
		gMainbus->SetDevice(i, gBusErr);
	}

	// 少ないので手動で置いていく。
#define A(addr)	(((addr) >> 24) & 0x3f)
	gMainbus->SetDevice(A(0xe0'000000), gNewsIO);
	gMainbus->SetDevice(A(0xe1'000000), gNewsCtlr);
	gMainbus->SetDevice(A(0xf8'000000), gPROM);
}

// デストラクタ
VM_NEWS::~VM_NEWS()
{
}

// 初期化
bool
VM_NEWS::Init()
{
	// 親クラス
	if (inherited::Init() == false) {
		return false;
	}

	// RAM 容量が確定したので、ここで devtable に RAM を配置。
	int ram_size_MB = gMainRAM->GetSizeMB();
	int i = 0;
	while (ram_size_MB >= 16) {
		// 16MB 単位にとりあえずしとく
		gMainbus->SetDevice(i++, gMainRAM);
		ram_size_MB -= 16;
	}

	// $40'000000〜、$80'000000〜、$c0'000000〜はミラーっぽい。
	gMainbus->CopyDevtable(0x40, 0, 0x40);
	gMainbus->CopyDevtable(0x80, 0, 0x40);
	gMainbus->CopyDevtable(0xc0, 0, 0x40);

	return true;
}
